# frozen_string_literal: true

require 'role_management/actions/create_role_document'
require 'role_management/models'
require 'role_management/errors'

require 'cht-lib-fabric/contexts/request'
require 'cht-lib-fabric/crns/tenant'

require 'role_management/helpers/flow_down_helper'

require 'role_management/models/privileges'

RSpec.describe ::RoleManagement::Actions::CreateRoleDocument do
  include_context 'kafka'
  include ::RoleManagement::Models::Privileges

  before { Application.stub('clients.authorization', TrueAuthorizer.new) }
  after { Application.unstub('clients.authorization') }

  let(:request) {
    req = ::Platform::Contexts::Request.new
    req.client_ip = IPAddr.new('1.1.1.1')
    req.user_agent = 'Mozilla/4.0 WebTV/2.6 (compatible; MSIE 4.0)'
    req.current_customer = customer_id
    req.current_user = user_id
    req
  }

  let(:jwt_subject) { "crn:1:user/1234" }
  let(:jwt_claims) { { sub: jwt_subject } }

  before(:each) do
    allow(request).to receive(:jwt_claims).and_return(jwt_claims)
  end

  before(:each) do
    allow_any_instance_of(::RoleManagement::Helpers::FlowDownHelper).to\
      receive(:enable_flow_down).and_return([])
  end

  let(:ctx) { {request: request} }

  describe '#call' do
    subject do
      described_class.new.call(name, description, restricted, permissions, nil, ctx)
    end
    let(:customer_id) { 1 }
    let(:user_id) { 3871 }
    let(:name) { "Fake name" }
    let(:description) { "Fake description" }
    let(:restricted) { false }
    let(:organization_id) { 17 }

    describe 'create role document with valid permissions' do
      let(:permissions) { [ {"action" => "read", "subject" => "custom_report"},
                            {"action" => "update", "subject" => "custom_report"} ] }

      it 'responds with the CRN of the new role document' do
        result, metadata = subject

        expect(result.id).to include("crn:#{customer_id}:role-document/")
        expect(result.name).to eq(name)
        expect(result.description).to eq(description)
        expect(result.restricted).to eq(false)
        expect(result.permissions.count).to eq(permissions.count)
      end
    end

    describe 'create role document when the caller is a service account' do
      let(:jwt_subject) { "crn::service-account/super-service" }

      let(:permissions) { [ {"action" => "read", "subject" => "custom_report"},
                            {"action" => "update", "subject" => "custom_report"} ] }

      it 'responds with the CRN of the new role document' do
        result, metadata = subject

        expect(result.id).to include("crn:#{customer_id}:role-document/")
        expect(result.name).to eq(name)
        expect(result.description).to eq(description)
        expect(result.restricted).to eq(false)
        expect(result.permissions.count).to eq(permissions.count)
      end
    end

    describe 'create a restricted role document' do
      let(:permissions) { [ {"action" => "read", "subject" => "custom_report"},
                            {"action" => "update", "subject" => "custom_report"} ] }
      let(:restricted) { true }

      it 'responds with the CRN of the new role document' do
        result, metadata = subject

        expect(result.id).to include("crn:#{customer_id}:role-document/")
        expect(result.name).to eq(name)
        expect(result.restricted).to eq(true)
        expect(result.description).to eq(description)
        expect(result.permissions.count).to eq(permissions.count)
        expect(result.metadata).to be_truthy
      end
    end

    describe 'create role document with invalid permission' do
      let(:permissions) { [{"action" => "read", "subject" => "foo"}] }

      it 'raises a ::Platform::GraphQL::Errors::ExecutionError' do
        expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError)
      end
    end

    describe 'when the user is not authorized to create role documents' do
      before { Application.stub('clients.authorization', FalseAuthorizer.new) }
      let(:permissions) { [{"action" => "read", "subject" => "foo"}] }

      it 'raises a ::Platform::GraphQL::Errors::ExecutionError with a not_authorized error' do
        expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Not authorized")
      end
    end

    describe 'create role document with valid organization' do
      let(:permissions) { [ {"action" => "read", "subject" => "custom_report"},
        {"action" => "update", "subject" => "custom_report"} ] }
      let(:request) {
        req = ::Platform::Contexts::Request.new
        req.client_ip = IPAddr.new('1.1.1.1')
        req.user_agent = 'Mozilla/4.0 WebTV/2.6 (compatible; MSIE 4.0)'
        req.current_customer = customer_id
        req.current_user = user_id
        req.current_organization = organization_id
        req
      }

      it 'responds with the attributes of the new role document' do
        result, metadata  = subject

        expect(result.id).to include("crn:#{customer_id}:role-document/")
        expect(result.name).to eq(name)
        expect(result.description).to eq(description)
        expect(result.permissions.count).to eq(permissions.count)
        # We expect the raw org_id here
        expect(result.organizationId).to eq(organization_id)
        expect(result.metadata).to be_truthy
      end
    end

    describe 'create role document with blank name' do

      let(:name) { "  " }
      let(:permissions) { [ {"action" => "read", "subject" => "custom_report"},
                            {"action" => "update", "subject" => "custom_report"} ] }

      it 'raises a ::Platform::GraphQL::Errors::ExecutionError with missing_role_document_name error' do
        expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Missing role document name")
      end
    end

    describe 'create a role document with all privileges' do
      let(:permissions) { all_privileges.map { |item| { "action"=>"#{item[0]}", "subject"=>"#{item[1]}" } }}
      it 'should successfully create the role document' do
        expect { subject }.to_not raise_error
      end
    end
  end
end
