# frozen_string_literal: true

require 'role_management/actions/create_role_document'
require 'role_management/models'
require 'role_management/errors'

require 'cht-lib-fabric/contexts/request'
require 'cht-lib-fabric/crns/tenant'

require 'role_management/helpers/flow_down_helper'

RSpec.describe ::RoleManagement::Actions::DeleteRoleDocument do
  include_context 'kafka'

  before { Application.stub('clients.authorization', TrueAuthorizer.new) }
  after { Application.unstub('clients.authorization') }

  let(:request) do
    req = ::Platform::Contexts::Request.new
    req.client_ip = IPAddr.new('1.1.1.1')
    req.user_agent = 'Mozilla/4.0 WebTV/2.6 (compatible; MSIE 4.0)'
    req.current_customer = customer_id
    req.current_user = user_id
    req.current_organization = organization_id
    req
  end

  let(:jwt_subject) { "crn:1:user/17" }
  let(:jwt_claims) { { sub: jwt_subject } }

  before do
    allow(request).to receive(:jwt_claims).and_return(jwt_claims)
  end

  let(:ctx) { {request: request} }

  describe '#call' do
    subject do
      described_class.new.call(role_doc_id, ctx)
    end
    let(:customer_id) { 1 }
    let(:user_id) { 17 }
    let(:organization_id) { 17 }
    let(:customer_crn) do
      ::Platform::CRNs::Tenant.new(customer: customer_id)
    end
    let(:user_crn) do
      ::Platform::CRNs::User.new(customer: customer_id, user: 17)
    end

    # Create a user defined role document to delete
    let(:role_document) {
      FactoryBot.create(:role_document,
                        name: "RoleDocument 1",
                        customer_crn: customer_crn.to_s,
                        created_by_crn: user_crn.to_s,
                        updated_by_crn: user_crn.to_s,
                        system_defined: false,
                        is_deleted: false,
                        organization_id: organization_id)
    }

    describe 'delete role document with no sharing' do
      let(:role_doc_id) do
        "crn:#{customer_id}:role-document/#{role_document.id}"
      end
      before(:each) do
        expect_any_instance_of(::RoleManagement::Helpers::FlowDownHelper).not_to\
          receive(:delete_flow_down)
        allow_any_instance_of(::RoleManagement::Helpers::FlowDownHelper).to\
          receive(:get_flow_down_items).and_return([])
        allow_any_instance_of(::RoleManagement::Helpers::FlowDownHelper).to\
          receive(:get_flow_down_items_by_org).and_return([])
      end
      it 'responds with the CRN of the deleted role document and should not call flow down ' do
        result = subject
        expect(result).to include("crn:#{customer_id}:role-document/")
      end
    end

    describe 'delete role document with sharing' do
      let(:role_doc_id) do
        "crn:#{customer_id}:role-document/#{role_document.id}"
      end
      before(:each) do
        expect_any_instance_of(::RoleManagement::Helpers::FlowDownHelper).to\
          receive(:delete_flow_down).and_return([])
        allow_any_instance_of(::RoleManagement::Helpers::FlowDownHelper).to\
          receive(:get_flow_down_items).and_return([role_document.id])
        allow_any_instance_of(::RoleManagement::Helpers::FlowDownHelper).to\
          receive(:get_flow_down_items_by_org).and_return([])
      end
      it 'responds with the CRN of the deleted role document and should call flow down ' do
        result = subject
        expect(result).to include("crn:#{customer_id}:role-document/")
      end
    end
  end
end
