# frozen_string_literal: true

require 'role_management/actions/node'
require 'role_management/models'
require 'cht-lib-fabric/contexts/request'
require 'cht-lib-fabric/crns'
require 'cht-lib-fabric/crns/tenant'
require 'role_management/helpers/flow_down_helper'

RSpec.describe ::RoleManagement::Actions::Node do
  before { Application.stub('clients.authorization', TrueAuthorizer.new) }
  after { Application.unstub('clients.authorization') }

  let(:request) {
    req = ::Platform::Contexts::Request.new
    req.client_ip = IPAddr.new('1.1.1.1')
    req.user_agent = 'Mozilla/4.0 WebTV/2.6 (compatible; MSIE 4.0)'
    req.current_customer = customer_id
    req.current_organization = current_org_id

    req
  }

  let(:jwt_subject) { "crn:1:user/1234" }
  let(:jwt_claims) { { sub: jwt_subject } }

  before do
    allow(request).to receive(:jwt_claims).and_return(jwt_claims)
  end

  before(:each) do
    allow_any_instance_of(::RoleManagement::Helpers::FlowDownHelper).to\
      receive(:get_flow_down_items_by_org).and_return([])
    allow_any_instance_of(::RoleManagement::Helpers::FlowDownHelper).to\
      receive(:get_flow_down_items).and_return([])      
  end

  let(:ctx) { {request: request} }

  describe '#call' do
    let(:customer_id) { 1 }
    let(:current_org_id) { 17 }
    let(:current_org_crn) { ::Platform::CRNs::Organization.new(customer: customer_id,
                                                               details: current_org_id.to_s)
    }
    let(:role_document_crn) { ::RoleManagement::Models::CRNs::RoleDocument.from_s("crn::role-document/1") }
    let(:system_role_doc) {
      ::RoleManagement::Stores::RoleDocument.where(id: role_document_crn.details.to_i,
                                                   system_defined: true)
        .first
    }

    context 'when given a crn for a role document' do
      subject do
        described_class.new.call(role_document_crn, ctx)
      end
      context 'that exists' do
        describe 'is system defined' do
          it 'returns the system defined role document node' do
            result = subject
            expect(result.id).to eq(role_document_crn.to_s)
            expect(result.name).to eq(system_role_doc.name)
            expect(result.description).to eq(system_role_doc.description)
            expect(result.system_defined).to be true
            expect(result.created_at).to eq(system_role_doc.created_at)
            expect(result.updated_at).to eq(system_role_doc.created_at)
            expect(result.permissions).to be_empty
          end
        end

        describe 'is customer defined' do
          let(:customer_crn) {
            ::Platform::CRNs::Tenant.new(customer: customer_id)
          }
          let(:user_crn) {
            ::Platform::CRNs::User.new(user: 3871, customer: customer_id)
          }
          let!(:customer_role_doc) { FactoryBot.create(:role_document,
                                                    name: "To be queried",
                                                    customer_crn: customer_crn.to_s,
                                                    created_by_crn: user_crn.to_s,
                                                    updated_by_crn: user_crn.to_s,
                                                    system_defined: false,
                                                    is_deleted: false,
                                                    organization_id: current_org_id) }
          let(:role_document_crn) {
            ::RoleManagement::Models::CRNs::RoleDocument.new(customer: customer_id,
                                                             details: customer_role_doc.id.to_s)
          }
          it 'returns the customer defined role document node' do
            node = subject
            expect(node.id).to eq(role_document_crn.to_s)
            expect(node.name).to eq(customer_role_doc.name)
            expect(node.description).to eq(customer_role_doc.description)
            expect(node.system_defined).to be false
            expect(node.created_at).to eq(customer_role_doc.created_at)
            expect(node.updated_at).to eq(customer_role_doc.created_at)
            expect(node.permissions).to be_empty
          end
        end
        describe 'is customer defined and shared' do
          let(:customer_crn) {
            ::Platform::CRNs::Tenant.new(customer: customer_id)
          }
          let(:user_crn) {
            ::Platform::CRNs::User.new(user: 3871, customer: customer_id)
          }
          let!(:shared_role_doc) { FactoryBot.create(:role_document,
                                                    name: "Shared Role Document",
                                                    customer_crn: customer_crn.to_s,
                                                    created_by_crn: user_crn.to_s,
                                                    updated_by_crn: user_crn.to_s,
                                                    system_defined: false,
                                                    is_deleted: false,
                                                    organization_id: current_org_id) }
          let(:role_document_crn) {
            ::RoleManagement::Models::CRNs::RoleDocument.new(customer: customer_id,
                                                             details: shared_role_doc.id.to_s)
          }
          before(:each) do
            allow_any_instance_of(::RoleManagement::Helpers::FlowDownHelper).to\
              receive(:get_flow_down_items_by_org).and_return([role_document_crn])
            expect_any_instance_of(::RoleManagement::Helpers::FlowDownHelper).not_to\
              receive(:get_flow_down_items)
          end
          it 'returns the customer defined and shared role document node' do
            node = subject
            expect(node.id).to eq(role_document_crn.to_s)
            expect(node.name).to eq(shared_role_doc.name)
            expect(node.description).to eq(shared_role_doc.description)
            expect(node.system_defined).to be false
            expect(node.created_at).to eq(shared_role_doc.created_at)
            expect(node.updated_at).to eq(shared_role_doc.created_at)
            expect(node.permissions).to be_empty
          end
        end
      end

      context 'that does not exist' do
        describe 'is system defined' do
          let(:role_document_crn) { ::RoleManagement::Models::CRNs::RoleDocument.from_s("crn::role-document/100")}

          it 'raises a ::Platform::GraphQL::Errors::ExecutionError' do
            expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError)
          end
        end

        describe 'is customer defined' do
          let(:role_document_crn) { ::RoleManagement::Models::CRNs::RoleDocument.from_s("crn:1:role-document/100") }

          it 'raises a ::Platform::GraphQL::Errors::ExecutionError' do
            expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError)
          end
        end
      end

      describe 'that exists in another organization' do
        let(:customer_crn) {::Platform::CRNs::Tenant.new(customer: customer_id)}
        let(:user_crn) {::Platform::CRNs::User.new(user: 3871, customer: customer_id)}

        let!(:doc_in_other_org) do
          FactoryBot.create(:role_document,
                            name: "role doc from another org",
                            customer_crn: customer_crn.to_s,
                            created_by_crn: user_crn.to_s,
                            updated_by_crn: user_crn.to_s,
                            system_defined: false,
                            is_deleted: false,
                            organization_id: current_org_id + 1)
        end

        let(:role_document_crn) {
          ::RoleManagement::Models::CRNs::RoleDocument.new(
            customer: customer_id,
            details: doc_in_other_org.id.to_s
          )
        }

        it 'raises a ::Platform::GraphQL::Errors::ExecutionError' do
          expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError)
        end

        describe 'and the caller is a service account' do
          let(:jwt_subject) { "crn::service-account/super-service" }
          it "should be found" do
            node = subject
            expect(node.name).to eq doc_in_other_org.name
            expect(node.organizationId).to eq doc_in_other_org.organization_id
          end
        end
      end
    end

    context 'when given a crn for a user group role binding' do
      subject do
        described_class.new.call(user_group_role_binding_crn, ctx)
      end

      context 'that exists' do
        let(:customer_crn) {
          ::Platform::CRNs::Tenant.new(customer: customer_id)
        }
        let(:user_crn) {
          ::Platform::CRNs::User.new(user: 3871, customer: customer_id)
        }
        let!(:ugrb) { FactoryBot.create(:user_group_role_binding,
                                        customer_crn: customer_crn.to_s,
                                        organization_crn: current_org_crn.to_s,
                                        created_by_crn: user_crn.to_s,
                                        updated_by_crn: user_crn.to_s,
                                        role_document: system_role_doc
                                       ) }
        let!(:user_group_role_binding_crn) { ugrb.crn }

        it 'returns the role binding node' do
          node = subject
          expect(node.id).to eq(ugrb.crn.to_s)
          expect(node.user_group_id).to eq(ugrb.user_group_crn)
          expect(node.organization_id).to eq(ugrb.organization_crn)
          expect(node.role_document.id).to eq(ugrb.role_document.crn.to_s)
        end
      end

      context 'that does not exist' do
        let!(:user_group_role_binding_crn) { ::Platform::CRN.from_s("crn::role-mgmt/ugrb:17171717") }

        it 'raises a ::Platform::GraphQL::Errors::ExecutionError' do
          expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError)
        end
      end
    end

    context 'security' do
      subject do
        described_class.new.call(role_document_crn, ctx)
      end
      describe 'when given a crn for another tenant' do
        let(:other_customer_id) { 2 }
        let(:other_customer_crn) {
          ::Platform::CRNs::Tenant.new(customer: other_customer_id)
        }
        let(:user_crn) {
          ::Platform::CRNs::User.new(user: 3872, customer: other_customer_id)
        }
        let!(:other_customer_role_doc) { FactoryBot.create(:role_document,
                                                  name: "Other customers role doc",
                                                  customer_crn: other_customer_crn.to_s,
                                                  created_by_crn: user_crn.to_s,
                                                  updated_by_crn: user_crn.to_s,
                                                  system_defined: false,
                                                  is_deleted: false) }
        let(:role_document_crn) {
          ::RoleManagement::Models::CRNs::RoleDocument.new(customer: other_customer_id,
                                                           details: other_customer_role_doc.id.to_s)

        }

        it 'raises a ::Platform::GraphQL::Errors::ExecutionError' do
          expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError)
        end
      end

      describe 'when the user is not authorized to create role documents' do
        before { Application.stub('clients.authorization', FalseAuthorizer.new) }

        it 'raises a ::Platform::GraphQL::Errors::ExecutionError with a not_authorized error' do
          expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Not authorized")
        end
      end
    end
  end
end
