# frozen_string_literal: true

require 'role_management/actions/permission_groups'
require 'role_management/models/privileges'
require 'role_management/models'

RSpec.describe ::RoleManagement::Actions::PermissionGroups do

  let(:permissions) {
    Application['role_management.actions.permission_groups']
  }
  describe '#build_hierarchy' do
    describe 'when permission has no child' do

      let(:child_privis) {
        [
            RoleManagement::Models::Privileges::PrivilegeTuple.new(:read_owned, :customer),
            RoleManagement::Models::Privileges::PrivilegeTuple.new(:update_owned, :customer),
            RoleManagement::Models::Privileges::PrivilegeTuple.new(:delete_owned, :customer),
        ]
      }
      let(:priv_section_or_tuple) {
        RoleManagement::Models::Privileges::PrivilegeSection.new("Customer", false, child_privis, false)
      }

      let(:permission_to_crn) do
        { [:read_owned, :customer] => "crn::permission/1",
          [:update_owned, :customer] => "crn::permission/2",
          [:delete_owned, :customer] => "crn::permission/3"}
      end

      it 'returns permission group with blank childIds' do
        result = permissions.build_hierarchy(priv_section_or_tuple, permission_to_crn, Array.new, Set.new)
        expect(result.childIds).to be_empty
      end
    end

    describe 'when permission which has children' do

      let(:priv_section_or_tuple) {
        RoleManagement::Models::Privileges::PrivilegeSection.new("VMWare", false, child_privis, false)
      }

      let(:child_privis) {
        [
            RoleManagement::Models::Privileges::PrivilegeSection.new("Dashboard", false, dashboard_privis, false),
            RoleManagement::Models::Privileges::PrivilegeSection.new("Accounts", false, account_privis, false),
            RoleManagement::Models::Privileges::PrivilegeSection.new("Agents", false, agent_privis, false)
        ]
      }

      let(:dashboard_privis) {
        [
            RoleManagement::Models::Privileges::PrivilegeTuple.new(:read, :vmware_dashboard),
            RoleManagement::Models::Privileges::PrivilegeTuple.new(:change, :vmware_dashboard)
        ]
      }

      let(:account_privis) {
        [
            RoleManagement::Models::Privileges::PrivilegeTuple.new(:create, :vmware_account),
            RoleManagement::Models::Privileges::PrivilegeTuple.new(:read, :vmware_account),
            RoleManagement::Models::Privileges::PrivilegeTuple.new(:update, :vmware_account),
            RoleManagement::Models::Privileges::PrivilegeTuple.new(:delete, :vmware_account)
        ]
      }

      let(:agent_privis) {
        [
            RoleManagement::Models::Privileges::PrivilegeTuple.new(:read, :vmware_agent_configuration),
            RoleManagement::Models::Privileges::PrivilegeTuple.new(:update, :vmware_agent_configuration),
            RoleManagement::Models::Privileges::PrivilegeTuple.new(:enable_autoupdate, :vmware_agent_status),
            RoleManagement::Models::Privileges::PrivilegeTuple.new(:disable_autoupdate, :vmware_agent_status)
        ]
      }

      let(:permission_to_crn) do
        { [:read, :vmware_dashboard] => "crn::permission/1332",
          [:change, :vmware_dashboard] => "crn::permission/1333",
          [:create, :vmware_account] => "crn::permission/1354",
          [:read, :vmware_account] => "crn::permission/1355",
          [:update, :vmware_account] => "crn::permission/1356",
          [:delete, :vmware_account] => "crn::permission/1357",
          [:read, :vmware_agent_configuration] => "crn::permission/1364",
          [:update, :vmware_agent_configuration] => "crn::permission/1365",
          [:enable_autoupdate, :vmware_agent_status] => "crn::permission/1367",
          [:disable_autoupdate, :vmware_agent_status] => "crn::permission/1366"}
      end

      it 'returns permission group with non-empty childIds' do
        result = permissions.build_hierarchy(priv_section_or_tuple, permission_to_crn, Array.new, Set.new)
        expect(result.childIds.count).to eq 3
      end
    end
  end

  describe '#root_permissions' do

    describe 'when there are 2 root elements with children' do

      let(:all_permissions) {
        [root_element1, root_element2, child_11, child_12, child_21]
      }

      let(:root_element1) {
        ::RoleManagement::Models::PermissionGroup.new(
            groupName: "Pulses",
            permissionGroup: [],
            permissions: [],
            id: "4eef6ce96d93d8d62990fb62a4be9aff",
            childIds: ["374368f6b6c7149b97cb2f9f931a3b1a", "866680541deaeb1b4a78da3e6d3712e7"])
      }

      let(:child_11_permissions) {
        [::RoleManagement::Models::Permission.new(
            id: "crn::permission/6",
            action: "read",
            subject: "cost_pulse",
            label: "Read Cost Pulse"
        ), ::RoleManagement::Models::Permission.new(
             id: "crn::permission/7",
             action: "subscribe",
             subject: "cost_pulse",
             label: "Update Cost Pulse"
         )]
      }

      let(:child_11) {
        ::RoleManagement::Models::PermissionGroup.new(
            groupName: "Cost",
            permissionGroup: [],
            permissions: child_11_permissions,
            id: "374368f6b6c7149b97cb2f9f931a3b1a",
            childIds: [])
      }

      let(:child_12_permissions) {
        [::RoleManagement::Models::Permission.new(
            id: "crn::permission/9",
            action: "read",
            subject: "ri_utilization_pulse",
            label: "Read RI Utilization Pulse"
        ), ::RoleManagement::Models::Permission.new(
            id: "crn::permission/10",
            action: "subscribe",
            subject: "ri_utilization_pulse",
            label: "Subscribe RI Utilization Pulse"
        )]
      }

      let(:child_12) {
        ::RoleManagement::Models::PermissionGroup.new(
            groupName: "RI",
            permissionGroup: [],
            permissions: child_12_permissions,
            id: "866680541deaeb1b4a78da3e6d3712e7",
            childIds: [])
      }

      let(:root_element2) {
        ::RoleManagement::Models::PermissionGroup.new(
            groupName: "Reports",
            permissionGroup: [],
            permissions: [],
            id: "8230b8a494ccc2a4db4dbaa0664239ac",
            childIds: ["31c381bbfbf8f58a6ef7eb7d2a963dbc"])
      }

      let(:child_21_permissions) {
        [::RoleManagement::Models::Permission.new(
            id: "crn::permission/1195",
            action: "read",
            subject: "gcp_instance_usage_report",
            label: "Read GCP Instance Usage Report"
        ), ::RoleManagement::Models::Permission.new(
            id: "crn::permission/1196",
            action: "export",
            subject: "gcp_instance_usage_report",
            label: "Export GCP Instance Usage Report"
        ),::RoleManagement::Models::Permission.new(
            id: "crn::permission/1197",
            action: "save",
            subject: "gcp_instance_usage_report",
            label: "Save GCP Instance Usage Report"
        )]
      }

      let(:child_21) {
        ::RoleManagement::Models::PermissionGroup.new(
            groupName: "Gcp Instance Usage Report",
            permissionGroup: [],
            permissions: child_21_permissions,
            id: "31c381bbfbf8f58a6ef7eb7d2a963dbc",
            childIds: [])
      }

      let(:all_child_ids) {
        ["374368f6b6c7149b97cb2f9f931a3b1a", "866680541deaeb1b4a78da3e6d3712e7", "31c381bbfbf8f58a6ef7eb7d2a963dbc"]
      }

      it 'returns 2 permissions with isRoot as true ' do
        result = permissions.root_permissions(all_permissions, all_child_ids)

        #fetch the root permissions
        root_elements = []
        result.map do |permission|
          if(permission.isRoot)
            root_elements << permission
          end
        end

        expect(root_elements.count).to eq 2
      end
    end

  end

end