# frozen_string_literal: true

require 'role_management/graphql/schema'
require 'cht-lib-fabric/contexts/request'
require 'cht-lib-fabric/crns/tenant'

require 'role_management/helpers/flow_down_helper'

RSpec.describe ::RoleManagement::GraphQL::Schema do
  before { Application.stub('clients.authorization', TrueAuthorizer.new) }
  after { Application.unstub('clients.authorization') }

  let(:variables) { nil }

  let(:request) do
    req = ::Platform::Contexts::Request.new
    req.client_ip = IPAddr.new('1.1.1.1')
    req.user_agent = 'Mozilla/4.0 WebTV/2.6 (compatible; MSIE 4.0)'
    req.current_customer = 2
    req.current_user = 17
    req.current_organization = 17
    req
  end

  let(:jwt_subject) { "crn:1:user/1234" }
  let(:jwt_claims) { { sub: jwt_subject } }

  before do
    allow(request).to receive(:jwt_claims).and_return(jwt_claims)
  end

  before(:each) do
    allow_any_instance_of(::RoleManagement::Helpers::FlowDownHelper).to\
      receive(:get_flow_down_items).and_return([])
  end

  subject do
    RoleManagement::GraphQL::Schema.execute(
      graphql_query,
      context: { request: request },
      variables: variables
    )
  end

  describe 'it can execute a query for role documents' do
    let(:customer_crn) do
      ::Platform::CRNs::Tenant.new(customer: 2)
    end

    let(:user_crn) do
      ::Platform::CRNs::User.new(customer: 2, user: 17)
    end

    let(:organization_id) { 17 }

    let(:graphql_query) do
      <<~GRAPHQL
        query {
            roleDocuments {
                edges {
                    node {
                        id
                        name
                        description
                        createdByUserId
                        updatedByUserId
                        metadata {
                          shared {
                            enabled
                          }
                        }
                    }
                }
            }
        }
      GRAPHQL
    end

    # Create a user defined role document
    before(:each) do
      FactoryBot.create(:role_document,
                        name: "RoleDocument 1",
                        customer_crn: customer_crn.to_s,
                        created_by_crn: user_crn.to_s,
                        updated_by_crn: user_crn.to_s,
                        system_defined: false,
                        is_deleted: false,
                        organization_id: organization_id)
    end

    it 'returns the createdByUserId and updatedByUserId field' do
      role_documents = subject["data"]["roleDocuments"]["edges"]
      role_documents.each do |edge|
        expect(edge['node']['createdByUserId']).to be_truthy
        expect(edge['node']['updatedByUserId']).to be_truthy
      end
    end

    context 'for a user defined role document' do
      it 'createdByUserId and updatedByUserId field returns the id of the user that created it' do
        role_documents = subject["data"]["roleDocuments"]["edges"]
        expect(role_documents.last['node']['createdByUserId']).to eq(user_crn.to_s)
        expect(role_documents.last['node']['updatedByUserId']).to eq(user_crn.to_s)
      end
    end

    context 'for meta data on a role document' do
      it 'should return default metadata even if sharing isnt enabled if its requested' do
        role_documents = subject["data"]["roleDocuments"]["edges"]
        role_documents.each do |edge|
          expect(edge['node']['metadata']).to be_truthy
          expect(edge['node']['metadata']['shared']['enabled']).to be_falsey
        end
      end
    end
  end
end
