# frozen_string_literal: true

require 'role_management/actions/role_documents'
require 'role_management/models'

require 'cht-lib-fabric/contexts/request'
require 'cht-lib-fabric/crns/tenant'

require 'role_management/helpers/flow_down_helper'

RSpec.describe ::RoleManagement::Actions::RoleDocuments do
  before { Application.stub('clients.authorization', TrueAuthorizer.new) }
  after { Application.unstub('clients.authorization') }

  let(:request) {
    req = ::Platform::Contexts::Request.new
    req.client_ip = IPAddr.new('1.1.1.1')
    req.user_agent = 'Mozilla/4.0 WebTV/2.6 (compatible; MSIE 4.0)'
    req.current_customer = customer_id
    req.current_organization = 17
    req
  }

  let(:jwt_subject) { "crn:1:user/1234" }
  let(:jwt_claims) { { sub: jwt_subject } }

  before do
    allow(request).to receive(:jwt_claims).and_return(jwt_claims)
  end

  before(:each) do
    allow_any_instance_of(::RoleManagement::Helpers::FlowDownHelper).to\
      receive(:get_flow_down_items).and_return([])
  end

  let(:ctx) { {request: request} }

  describe '#call' do
    subject do
      described_class.new.call(ctx)
    end

    let(:customer_id) { 1 }
    let(:system_role_doc) {
      ::RoleManagement::Stores::RoleDocument.where(system_defined: true)
    }

    describe 'when no customer role documents defined' do
      it 'returns all system defined role documents' do
        result = subject
        expect(result).not_to be_empty
        expect(result.count).to eq(system_role_doc.count)

        result.each do |role_doc|
          # all role doc nodes are system defined
          expect(role_doc.system_defined).to eq true
        end
      end
    end

    describe 'when customer role documents are defined' do
      let(:customer_crn) {
        ::Platform::CRNs::Tenant.new(customer: customer_id)
      }
      let(:user_crn) {
        ::Platform::CRNs::Tenant.new(user: 3871, customer: customer_id)
      }
      let(:organization_id) { 17 }
      let(:customer_role_docs) {
        ::RoleManagement::Stores::RoleDocument.where(customer_crn: customer_crn.to_s,
                                                     system_defined: false,
                                                     organization_id: organization_id)
      }

      before(:each) do
        (0...4).each do |idx|
          FactoryBot.create(:role_document,
                            name: "To be queried #{idx}",
                            customer_crn: customer_crn.to_s,
                            created_by_crn: user_crn.to_s,
                            updated_by_crn: user_crn.to_s,
                            system_defined: false,
                            is_deleted: false,
                            organization_id: organization_id)
        end
      end


      it 'returns all system and customer defined role documents' do
        result = subject
        expect(result).not_to be_empty
        expect(result.count).to eq(system_role_doc.count +
                                                customer_role_docs.count)

        system_role_doc_results = result.select do |role_doc|
          role_doc.system_defined
        end
        expect(system_role_doc_results.count).to eq(system_role_doc.count)

        customer_role_doc_results = result.select do |role_doc|
          !role_doc.system_defined
        end
        expect(customer_role_doc_results.count).to eq(customer_role_docs.count)
      end

      context 'for an org that doesnt exist' do
        let(:organization_id) { 9999999 }
        it 'returns all system defined role documents and no customer defined for non-existing organization' do
          result = subject

          expect(result).not_to be_empty
          expect(result.count).to eq(system_role_doc.count)

          sytem_role_doc_results = result.select do |role_doc_edge|
            role_doc_edge.system_defined
          end
          expect(sytem_role_doc_results.count).to eq(system_role_doc.count)

          customer_role_doc_results = result.select do |role_doc_edge|
            !role_doc_edge.system_defined
          end
          expect(customer_role_doc_results.count).to eq(0)
        end
      end

      context 'security' do
        let(:other_customer_id) { 2 }
        let(:other_customer_crn) {
          ::Platform::CRNs::Tenant.new(customer: other_customer_id)
        }
        let(:other_user_crn) {
          ::Platform::CRNs::Tenant.new(user: 3872, customer: other_customer_id)
        }
        let(:organization_id) { 17 }
        before(:each) do
          (0...2).each do |idx|
            FactoryBot.create(:role_document,
                              name: "To be queried #{idx}",
                              customer_crn: other_customer_crn.to_s,
                              created_by_crn: other_user_crn.to_s,
                              updated_by_crn: other_user_crn.to_s,
                              system_defined: false,
                              is_deleted: false,
                              organization_id: organization_id)
          end
        end

        it 'does not return the other customer"\'"s role documents' do
          result = subject

          result.each do |role_doc_edge|
            crn = ::Platform::CRN.from_s(role_doc_edge.id)
            # The role document CRNs returned NEVER belong to another customer
            expect(crn.customer).not_to eq(other_customer_id)
          end

          sytem_role_doc_results = result.select do |role_doc_edge|
            role_doc_edge.system_defined
          end
          expect(sytem_role_doc_results.count).to eq(system_role_doc.count)

          customer_role_doc_results = result.select do |role_doc_edge|
            !role_doc_edge.system_defined
          end
          expect(customer_role_doc_results.count).to eq(customer_role_docs.count)
        end
      end
    end

    describe 'when the user is not authorized to create role documents' do
      before { Application.stub('clients.authorization', FalseAuthorizer.new) }

      it 'raises a ::Platform::GraphQL::Errors::ExecutionError with a not_authorized error' do
        expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Not authorized")
      end
    end

    describe 'when looking up role documents' do
      let(:customer_crn) {
        ::Platform::CRNs::Tenant.new(customer: customer_id)
      }
      let(:user_crn) {
        ::Platform::CRNs::Tenant.new(user: 3871, customer: customer_id)
      }
      let(:organization_id) { 17 }
      let(:customer_role_docs) {
        ::RoleManagement::Stores::RoleDocument.where(customer_crn: customer_crn.to_s,
                                                     system_defined: false,
                                                     organization_id: organization_id)
      }

      before(:each) do
        (0...4).each do |idx|
          FactoryBot.create(:role_document,
                            name: "To be queried #{idx}",
                            customer_crn: customer_crn.to_s,
                            created_by_crn: user_crn.to_s,
                            updated_by_crn: user_crn.to_s,
                            system_defined: false,
                            is_deleted: false,
                            organization_id: organization_id)
        end
      end
      it 'returns all system and customer defined role documents with org ids as appropriate' do
        result = subject

        expect(result).not_to be_empty
        expect(result.count).to eq(system_role_doc.count +
                                                customer_role_docs.count)

        sytem_role_doc_results = result.select do |role_doc_edge|
          role_doc_edge.organizationId.nil? && role_doc_edge.system_defined
        end
        expect(sytem_role_doc_results.count).to eq(system_role_doc.count)

        customer_role_doc_results = result.select do |role_doc_edge|
          !role_doc_edge.organizationId.nil? && !role_doc_edge.system_defined
        end
        expect(customer_role_doc_results.count).to eq(customer_role_docs.count)
      end
    end

    describe 'when looking up role documents it should return shared role documents if they have been shared' do
      let(:customer_crn) {
        ::Platform::CRNs::Tenant.new(customer: customer_id)
      }
      let(:user_crn) {
        ::Platform::CRNs::Tenant.new(user: 3871, customer: customer_id)
      }
      let(:organization_id) { 17 }
      let(:parent_organization_id) { 16 }
      let(:customer_role_docs) {
        ::RoleManagement::Stores::RoleDocument.where(customer_crn: customer_crn.to_s,
                                                     system_defined: false,
                                                     organization_id: organization_id)
      }
      let(:customer_role_doc_id) {
        ::RoleManagement::Stores::RoleDocument.where(customer_crn: customer_crn.to_s,
                                                     system_defined: false,
                                                     organization_id: organization_id).first.id
      }

      let(:shared_role_docs) {
        ::RoleManagement::Stores::RoleDocument.where(customer_crn: customer_crn.to_s,
                                                     system_defined: false,
                                                     organization_id: parent_organization_id)
      }

      before(:each) do
        # Create a role document in a parent organization
        FactoryBot.create(:role_document,
                          name: "Shared Role Document",
                          customer_crn: customer_crn.to_s,
                          created_by_crn: user_crn.to_s,
                          updated_by_crn: user_crn.to_s,
                          system_defined: false,
                          is_deleted: false,
                          organization_id: parent_organization_id)

      end

      before(:each) do
        (0...4).each do |idx|
          FactoryBot.create(:role_document,
                            name: "To be queried #{idx}",
                            customer_crn: customer_crn.to_s,
                            created_by_crn: user_crn.to_s,
                            updated_by_crn: user_crn.to_s,
                            system_defined: false,
                            is_deleted: false,
                            organization_id: organization_id)
        end
      end

      # 22 is the Details portion of the Role Doc CRN
      before(:each) do
        allow_any_instance_of(::RoleManagement::Helpers::FlowDownHelper).to\
          receive(:get_flow_down_items).and_return([customer_role_doc_id])
      end

      it 'returns all system and customer defined and shared role documents' do
        result = subject
        expect(result).not_to be_empty
        expect(result.count).to eq(system_role_doc.count +
                                                customer_role_docs.count +
                                                shared_role_docs.count)

      end
    end

    describe 'and the caller is a service account' do
      let(:jwt_subject) { "crn::service-account/super-service" }

      let(:customer_crn) {
        ::Platform::CRNs::Tenant.new(customer: customer_id)
      }
      let(:user_crn) {
        ::Platform::CRNs::Tenant.new(user: 3871, customer: customer_id)
      }
      let(:organization_id) { 17 }

      let!(:doc_in_other_org) do
        FactoryBot.create(:role_document,
                          name: "role doc from another org",
                          customer_crn: customer_crn.to_s,
                          created_by_crn: user_crn.to_s,
                          updated_by_crn: user_crn.to_s,
                          system_defined: false,
                          is_deleted: false,
                          organization_id: 34)
      end

      it "should return all role bindings from all organizations" do
        result = subject
        expect(result).not_to be_empty
        customer_role_docs = result.reject do |role_doc_edge|
          role_doc_edge.system_defined
        end

        expect(customer_role_docs.count).to eq(1)

        node = customer_role_docs.first
        expect(node.name).to eq doc_in_other_org.name
        expect(node.organizationId).to eq doc_in_other_org.organization_id
      end
    end
  end
end
