# frozen_string_literal: true

require 'role_management/actions/update_role_document'
require 'role_management/models'
require 'role_management/errors'

require 'cht-lib-fabric/contexts/request'
require 'cht-lib-fabric/crns/tenant'

require 'role_management/helpers/flow_down_helper'

RSpec.describe ::RoleManagement::Actions::UpdateRoleDocument do
  include_context 'kafka'

  before { Application.stub('clients.authorization', TrueAuthorizer.new) }
  after { Application.unstub('clients.authorization') }

  let(:request) {
    req = ::Platform::Contexts::Request.new
    req.client_ip = IPAddr.new('1.1.1.1')
    req.user_agent = 'Mozilla/4.0 WebTV/2.6 (compatible; MSIE 4.0)'
    req.current_customer = customer_id
    req.current_user = user_id
    req.current_organization = 17
    req
  }

  let(:jwt_subject) { "crn:1:user/1234" }
  let(:jwt_claims) { { sub: jwt_subject } }

  before do
    allow(request).to receive(:jwt_claims).and_return(jwt_claims)
  end

  before(:each) do
    allow_any_instance_of(::RoleManagement::Helpers::FlowDownHelper).to\
      receive(:update_flow_down).and_return([])
  end

  let(:ctx) { {request: request} }

  describe '#call' do
    subject do
      described_class.new.call(id, updated_name, updated_description, nil, updated_permissions, nil, ctx)
    end

    let(:customer_id) { 1 }
    let(:user_id) { 3871 }
    let(:updated_name) { "Fake role doc" }
    let(:updated_description) { "Fake description for role doc" }
    let(:updated_permissions) { [ {"action" => "read", "subject" => "custom_report"},
                                  {"action" => "update", "subject" => "custom_report"} ] }

    let(:tenant_crn) {
      ::Platform::CRNs::Tenant.new(customer: customer_id)
    }
    let(:user_crn) {
      ::Platform::CRNs::User.new(user: user_id, customer: customer_id)
    }
    let(:organization_id) { 17 }

    let!(:existing_role_doc) { FactoryBot.create(:role_document,
                                                 name: "Existing role doc",
                                                 customer_crn: tenant_crn,
                                                 updated_by_crn: user_crn,
                                                 system_defined: false,
                                                 is_deleted: false,
                                                 organization_id: organization_id ) }

    context 'updating a role document' do
      describe 'that exists' do
        let(:id) {
          ::RoleManagement::Models::CRNs::RoleDocument.new(customer: customer_id,
                                                           details: existing_role_doc.id.to_s,
                                                           organization_id: organization_id)
                                                      .to_s
        }

        it 'successfully updates context with new role document attributes' do
          result, metadata = subject

          expected_id = ::RoleManagement::Models::CRNs::RoleDocument.new(customer: customer_id,
                                                                         details: existing_role_doc.id.to_s,
                                                                         organization_id: organization_id)
                                                                    .to_s
          expect(result.id).to eq(expected_id)
          expect(result.name).to eq(updated_name)
          expect(result.description).to eq(updated_description)
          expect(result.permissions.count).to eq(updated_permissions.count)
          expect(result.organizationId).to eq(organization_id)
          expect(result.metadata).to be_truthy
        end
      end

      describe 'that does not exist' do
        let(:id) {
          ::RoleManagement::Models::CRNs::RoleDocument.new(customer: customer_id,
                                                           details: "9999999",
                                                           organization_id: organization_id)
                                                      .to_s
        }

        it 'raises a ::Platform::GraphQL::Errors::ExecutionError' do
          expect { subject }.to raise_error { |error|
            expect(error).to be_a(::Platform::GraphQL::Errors::ExecutionError)
            expect(error.extensions["error"]).to eq(::RoleManagement::ErrorCodes::NOT_FOUND)
          }
        end
      end

      describe 'that refers to an org that does not exist' do
        let(:id) {
          ::RoleManagement::Models::CRNs::RoleDocument.new(customer: customer_id,
                                                           details: "9999999",
                                                           organization_id: 999999999)
                                                      .to_s
        }

        it 'raises a ::Platform::GraphQL::Errors::ExecutionError' do
          expect { subject }.to raise_error { |error|
            expect(error).to be_a(::Platform::GraphQL::Errors::ExecutionError)
            expect(error.extensions["error"]).to eq(::RoleManagement::ErrorCodes::NOT_FOUND)
          }
        end
      end

      describe 'with invalid name' do
        let(:id) {
          ::RoleManagement::Models::CRNs::RoleDocument.new(customer: customer_id,
                                                           details: existing_role_doc.id.to_s,
                                                           organization_id: organization_id)
              .to_s
        }
        let(:updated_name) { "" }

        it 'raises a ::Platform::GraphQL::Errors::ExecutionError with missing_role_document_name error' do
          expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Missing role document name")
        end
      end
    end

    context 'security' do
      describe 'when given a crn for another tenant' do
        let(:other_customer_id) { 2 }
        let(:other_customer_crn) {
          ::Platform::CRNs::Tenant.new(customer: other_customer_id)
        }
        let(:other_user_crn) {
          ::Platform::CRNs::User.new(user: 3872, customer: other_customer_id)
        }
        let!(:other_customer_role_doc) { FactoryBot.create(:role_document,
                                                  name: "Other customers role doc",
                                                  customer_crn: other_customer_crn.to_s,
                                                  created_by_crn: other_user_crn.to_s,
                                                  updated_by_crn: other_user_crn.to_s,
                                                  system_defined: false,
                                                  is_deleted: false,
                                                  organization_id: organization_id) }
        let(:id) {
          ::RoleManagement::Models::CRNs::RoleDocument.new(customer: other_customer_id,
                                                           details: other_customer_role_doc.id.to_s)
                                                      .to_s
        }

        it 'raises a ::Platform::GraphQL::Errors::ExecutionError' do
          expect { subject }.to raise_error { |error|
            expect(error).to be_a(::Platform::GraphQL::Errors::ExecutionError)
            expect(error.extensions["error"]).to eq(::RoleManagement::ErrorCodes::NOT_FOUND)
          }
        end
      end

      describe 'when given a crn for a system defined role document' do
        let(:system_defined_role_doc) {
          ::RoleManagement::Stores::RoleDocument.where(system_defined: true).first
        }
        let(:id) {
          ::RoleManagement::Models::CRNs::RoleDocument.new(customer: 0,
                                                           details: system_defined_role_doc.id.to_s)
                                                      .to_s
        }

        it 'raises a ::Platform::GraphQL::Errors::ExecutionError' do
          expect { subject }.to raise_error { |error|
            expect(error).to be_a(::Platform::GraphQL::Errors::ExecutionError)
            expect(error.extensions["error"]).to eq(::RoleManagement::ErrorCodes::NOT_FOUND)
          }
        end
      end
    end

    describe 'when the user is not authorized to create role documents' do
      before { Application.stub('clients.authorization', FalseAuthorizer.new) }
      let(:id) { "" }
      it 'raises a ::Platform::GraphQL::Errors::ExecutionError with a not_authorized error' do
        expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Not authorized")
      end
    end
  end
end
