# frozen_string_literal: true

require 'role_management/actions/update_user_group_role_bindings'
require 'role_management/models'

require 'cht-lib-fabric/contexts/request'
require 'cht-lib-fabric/crns'

require 'active_support/core_ext/hash/indifferent_access'

RSpec.describe ::RoleManagement::Actions::UpdateUserGroupRoleBindings do
  before { Application.stub('clients.authorization', TrueAuthorizer.new) }
  after { Application.unstub('clients.authorization') }

  let(:request) {
    req = ::Platform::Contexts::Request.new
    req.client_ip = IPAddr.new('1.1.1.1')
    req.user_agent = 'Mozilla/4.0 WebTV/2.6 (compatible; MSIE 4.0)'
    req.current_customer = customer_id
    req.current_organization = current_org_id
    req.current_user = user_id
    req
  }

  let(:jwt_subject) { "crn:1:user/1234" }
  let(:jwt_claims) { { sub: jwt_subject } }
  let(:ctx) { {request: request} }

  before do
    allow(request).to receive(:jwt_claims).and_return(jwt_claims)
  end

  let(:customer_id) { 1 }
  let(:user_id) { 3871 }
  let(:customer_crn) { ::Platform::CRNs::Tenant.new(customer: customer_id) }
  let(:user_crn) { ::Platform::CRNs::User.new(user: user_id, customer: customer_id) }
  let(:user_group_crn) { ::Platform::CRN.new(customer: customer_id, service: 'user-group', details: '1') }
  let(:user_group_no_sia_users) {
    {
      "node": {
        "__typename": "UserGroup",
        "id": "crn:2017:user-group/1",
        "name": "My Test App",
        "users": {
          "edges": []
        }
      }
    }
  }
  let(:user_group_sia_users) {
    {
      "node": {
        "__typename": "UserGroup",
        "id": "crn:2017:user-group/1000",
        "name": "My Test App",
        "users": {
          "edges": [{
            "node": {
              "id": "crn:2017:user/313312",
              "name": "My Test App",
              "email": "mytestapp-service-integration-user-2017@vmware.com",
              "serviceAccount": true,
              "oauthAppId": "91011"
            }
          }]
        }
      }
    }
  }

  let(:current_org_id) { 17 }
  let(:current_org_crn) { ::Platform::CRNs::Organization.new(
                                 customer: customer_id,
                                 details: current_org_id.to_s)
  }
  let(:target_org1_crn) { ::Platform::CRNs::Organization.new(
                                 customer: customer_id,
                                 details: (current_org_id + 1).to_s)
  }
  let(:target_org2_crn) { ::Platform::CRNs::Organization.new(
                                 customer: customer_id,
                                 details: (current_org_id + 2).to_s)
  }
  let(:role_doc) { ::RoleManagement::Stores::RoleDocument.where(id: 1, system_defined: true).first }
  let(:role_doc_crn) { role_doc.crn }

  # sample environment config
  let(:configuration) do
    {
      "serviceIntegrationUsers": {
        "edges": [{
          "node": {
            "id": "crn::service-integration/91011",
            "name": "TMC",
            "description": "This is the TMC Service Integration account",
            "email": "tmc-service-integration-user-2017@vmware.com",
            "approved": false,
            "approvedOn": nil,
            "approvedBy": nil,
            "roleDocumentId": "crn:2017:role-document/2"
          }
        }]
      }
    }.with_indifferent_access
  end


  before :each do
    allow_any_instance_of(::RoleManagement::Clients::UserManagement)
        .to receive(:get_user_group)
                .and_return(user_group_no_sia_users)
  end

  before :each do
    allow_any_instance_of(::RoleManagement::Clients::OrgManagement)
        .to receive(:get_organization)
                .and_return(target_org1_crn)
  end

  before :each do
    allow_any_instance_of(::RoleManagement::Clients::UserManagement)
        .to receive(:get_service_integration_accounts)
                .and_return(configuration)
  end

  describe '#call' do
    subject { described_class.new.call(request_params, ctx) }
    let(:request_params) { {} }

    describe 'with no actions' do
      it 'should return a successful, but blank response' do
          result = subject
          expect(result[:creates]).to be_nil
          expect(result[:updates]).to be_nil
          expect(result[:deletes]).to be_nil
      end
    end

    describe 'when creating role bindings' do
      let(:request_params) do
        { :creates => [
            { :user_group => {:id => user_group_crn.to_s },
              :role_document => {:id => role_doc_crn.to_s},
              :organizations => [{:id => target_org1_crn.to_s}, {:id => target_org2_crn.to_s}]
            }
          ]
        }
      end

      describe 'with an invalid role document' do
        let(:role_doc_crn) { "crn::role-document/9999999" }
        it 'should respond with an error' do
          expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Invalid role document")
        end
      end

      describe 'with an invalid user-group' do
        let(:user_group_crn) { ::Platform::CRN.new(customer: customer_id, service: 'user-group', details: '1000') }
        before :each do
          allow_any_instance_of(::RoleManagement::Clients::UserManagement)
              .to receive(:get_user_group)
                      .and_return(nil)
        end
        it 'should respond with an error' do
          expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Invalid user group")
        end
      end

      describe 'with an invalid user-group for an sia account pointing to a different role document' do
        let(:user_group_crn) { ::Platform::CRN.new(customer: customer_id, service: 'user-group', details: '1000') }
        before :each do
          allow_any_instance_of(::RoleManagement::Clients::UserManagement)
              .to receive(:get_user_group)
                      .and_return(user_group_sia_users)
        end
        it 'should respond with an error' do
          expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Invalid user group")
        end
      end

      describe 'with an invalid organization' do

        before :each do
          allow_any_instance_of(::RoleManagement::Clients::OrgManagement)
              .to receive(:get_organization)
                      .and_return(nil)
        end

        it 'should respond with an error' do
          expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Invalid organization")
        end
      end

      describe 'that do not already exist' do
        it 'should create the role binding' do
          result = subject
          expect(result['creates']).not_to be_empty
          ugrb = subject['creates'].first
          expect(ugrb.created_by_id).to eq(user_crn.to_s)
          expect(ugrb.updated_by_id).to eq(user_crn.to_s)
          expect(ugrb.organization_ids).to eq([target_org1_crn.to_s, target_org2_crn.to_s])
          expect(ugrb.organization_id).to eq(current_org_crn.to_s)
          expect(ugrb.role_document.id).to eq(role_doc_crn.to_s)
        end
      end

      describe 'that already exist' do
        before do
          # create conflicting ugrb
          FactoryBot.create(:user_group_role_binding,
                            customer_crn: customer_crn.to_s,
                            organization_crn: current_org_crn.to_s,
                            created_by_crn: user_crn.to_s,
                            updated_by_crn: user_crn.to_s,
                            role_document: role_doc
                           )
        end

        it 'should fail with an already exists error' do
          expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Role binding already exists")
        end
      end
    end

    describe 'when updating role bindings' do
      let(:request_params) do
        { :updates => [
            { :id => ugrb_crn.to_s,
              :organization_ids => [target_org2_crn.to_s]
            }
          ]
        }
      end
      let(:ugrb_crn) { ::RoleManagement::Models::CRNs::UserGroupRoleBinding.new(customer: customer_id, role_binding_id: ugrb_id)}

      describe 'that do not already exist' do
        let(:ugrb_id) { 12 }
        it 'should respond with not found' do
          expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Record not found")
        end
      end

      describe 'that already exist' do
        let(:ugrb_id) { existing_ugrb.crn.user_group_role_binding_id }
        let(:the_org_crn) { current_org_crn }
        let(:the_customer_crn) { customer_crn }
        let(:existing_ugrb) do
          FactoryBot.create(:user_group_role_binding,
                            customer_crn: the_customer_crn,
                            organization_crn: the_org_crn.to_s,
                            created_by_crn: user_crn.to_s,
                            updated_by_crn: user_crn.to_s,
                            role_document: role_doc
                           )
        end

        describe 'and belong to a different organization' do
          let(:other_org_crn) { ::Platform::CRNs::Organization.new(
                                       customer: customer_id,
                                       details: (current_org_id + 1).to_s)
          }
          let(:the_org_crn) { other_org_crn }

          it 'should respond with not found' do
            expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Record not found")
          end
        end

        describe 'and belong to a different customer' do
          let(:other_org_crn) { ::Platform::CRNs::Organization.new(
                                       customer: customer_id,
                                       details: (current_org_id + 2).to_s)
          }
          let(:other_customer_crn) { ::Platform::CRNs::Tenant.new(customer: customer_id + 1) }
          let(:the_org_crn) { other_org_crn }
          let(:the_customer_crn) { other_customer_crn }

          it 'should respond with not found' do
            expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Record not found")
          end
        end

        it 'should update successfully' do
          result = subject
          expect(result['updates'].length).to eq(1)
          result = subject['updates'].first
          expect(result.created_by_id).to eq(user_crn.to_s)
          expect(result.updated_by_id).to eq(user_crn.to_s)
          expect(result.organization_ids).to eq([target_org2_crn.to_s])
          expect(result.organization_id).to eq(current_org_crn.to_s)
          expect(result.role_document.id).to eq(role_doc_crn.to_s)
        end
      end
    end

    describe 'when deleting and recreating the same user group role bindings' do
      let(:existing_ugrb) do
        FactoryBot.create(:user_group_role_binding,
                          customer_crn: customer_crn,
                          organization_crn: current_org_crn.to_s,
                          created_by_crn: user_crn.to_s,
                          updated_by_crn: user_crn.to_s,
                          role_document: role_doc
                         )
      end

      let(:request_params) do
        { :creates => [
            { :user_group => {:id => existing_ugrb.user_group_crn.to_s },
              :role_document => {:id => existing_ugrb.role_document_crn.to_s},
              :organizations => [{:id => current_org_crn.to_s}]
            }
          ],
          :deletes => [{ :id => existing_ugrb.crn.to_s }]
        }
      end

      it 'should not fail with an already exists error' do
        expect { subject }.not_to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Role binding already exists")
      end
    end

    describe 'when deleting role bindings' do
      let(:request_params) do
        { :deletes => [{ :id => ugrb_crn.to_s }] }
      end

      let(:ugrb_crn) { ::RoleManagement::Models::CRNs::UserGroupRoleBinding.new(customer: customer_id, role_binding_id: ugrb_id)}

      describe 'that do not already exist' do
        let(:ugrb_id) { 12 }
        it 'should respond with not found' do
          expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Record not found")
        end
      end

      describe 'that already exist' do
        let(:ugrb_id) { existing_ugrb.crn.user_group_role_binding_id }
        let(:the_org_crn) { current_org_crn }
        let(:the_customer_crn) { customer_crn }
        let(:existing_ugrb) do
          FactoryBot.create(:user_group_role_binding,
                            customer_crn: the_customer_crn,
                            organization_crn: the_org_crn.to_s,
                            created_by_crn: user_crn.to_s,
                            updated_by_crn: user_crn.to_s,
                            role_document: role_doc
                           )
        end

        describe 'and belong to a different organization' do
          let(:other_org_crn) { ::Platform::CRNs::Organization.new(
                                       customer: customer_id,
                                       details: (current_org_id + 1).to_s)
          }
          let(:the_org_crn) { other_org_crn }

          it 'should respond with not found' do
            expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Record not found")
          end
        end

        describe 'and belong to a different customer' do
          let(:other_org_crn) { ::Platform::CRNs::Organization.new(
                                       customer: customer_id,
                                       details: (current_org_id + 2).to_s)
          }
          let(:other_customer_crn) { ::Platform::CRNs::Tenant.new(customer: customer_id + 1) }
          let(:the_org_crn) { other_org_crn }
          let(:the_customer_crn) { other_customer_crn }

          it 'should respond with not found' do
            expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Record not found")
          end
        end

        it 'should delete the role binding' do
          result = subject
          expect(result['deletes'].length).to eq(1)
          expect(result['deletes'].first).to eq(ugrb_crn.to_s)
        end
      end
    end

    describe 'when the user is not authorized to create role documents' do
      before { Application.stub('clients.authorization', FalseAuthorizer.new) }

      it 'raises a ::Platform::GraphQL::Errors::ExecutionError with a not_authorized error' do
        expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Not authorized")
      end
    end
  end
end
