# frozen_string_literal: true

require 'role_management/actions/user_group_role_bindings'
require 'role_management/models'

require 'cht-lib-fabric/contexts/request'
require 'cht-lib-fabric/crns'
require 'cht-lib-fabric/crns/tenant'

RSpec.describe ::RoleManagement::Actions::UserGroupRoleBindings do
  before { Application.stub('clients.authorization', TrueAuthorizer.new) }
  after { Application.unstub('clients.authorization') }

  let(:request) {
    req = ::Platform::Contexts::Request.new
    req.client_ip = IPAddr.new('1.1.1.1')
    req.user_agent = 'Mozilla/4.0 WebTV/2.6 (compatible; MSIE 4.0)'
    req.current_customer = customer_id
    req.current_organization = current_org_id
    req
  }

  let(:jwt_subject) { "crn:1:user/1234" }
  let(:jwt_claims) { { sub: jwt_subject } }

  before do
    allow(request).to receive(:jwt_claims).and_return(jwt_claims)
  end

  let(:role_doc) { ::RoleManagement::Stores::RoleDocument.where(id: 1, system_defined: true).first }

  let(:ctx) { {request: request} }

  describe '#call' do
    subject do
      described_class.new.call(ctx)
    end
    let(:customer_id) { 1 }
    let(:customer_crn) { ::Platform::CRNs::Tenant.new(customer: customer_id) }
    let(:user_crn) { ::Platform::CRNs::User.new(user: 3871, customer: customer_id) }
    let(:current_org_id) { 17 }
    let(:current_org_crn) { ::Platform::CRNs::Organization.new(customer: customer_id,
                                                               details: current_org_id.to_s)
    }

    describe 'when role bindings exist in several customers and organizations' do
      let(:current_org_crn_1b) { ::Platform::CRNs::Organization.new(customer: customer_id,
                                                                    details: (current_org_id + 1).to_s)
      }

      let(:customer2_id) { customer_id + 1 }
      let(:customer2_crn) {
        ::Platform::CRNs::Tenant.new(customer: customer_id + 1)
      }
      let(:customer2_org_crn) {
        ::Platform::CRNs::Organization.new(customer: customer2_id,
                                           details: (current_org_id + 1).to_s
                                          )
      }
      let(:user_crn) {
        ::Platform::CRNs::User.new(user: 3871, customer: customer_id)
      }
      let!(:ugrb) { FactoryBot.create(:user_group_role_binding,
                                      customer_crn: customer_crn.to_s,
                                      organization_crn: current_org_crn.to_s,
                                      created_by_crn: user_crn.to_s,
                                      updated_by_crn: user_crn.to_s,
                                      role_document: role_doc
                                     ) }
      let!(:user_group_role_binding_crn) { ugrb.crn.to_s }

      let!(:other_ugrbs) do
        [
          FactoryBot.create(:user_group_role_binding,
                            customer_crn: customer_crn.to_s,
                            organization_crn: current_org_crn_1b.to_s,
                            created_by_crn: user_crn.to_s,
                            updated_by_crn: user_crn.to_s,
                            role_document: role_doc
                           ),
          FactoryBot.create(:user_group_role_binding,
                            customer_crn: customer2_crn.to_s,
                            organization_crn: customer2_org_crn.to_s,
                            created_by_crn: user_crn.to_s,
                            updated_by_crn: user_crn.to_s,
                            role_document: role_doc
                           )
        ]
      end

      it "returns all role bindings in the request's organization" do
        result = subject

        expect(result.length).to eq(1)
        expect(result.first.id).to eq(ugrb.crn.to_s)
      end

      describe 'and the caller is a service account' do
        let(:jwt_subject) { "crn::service-account/super-service" }
        it "should return all role bindings from all organizations" do
          result = subject
          expected_count = RoleManagement::Stores::UserGroupRoleBinding
                             .where(customer_crn: "crn:#{customer_id}:tenant/#{customer_id}").count

          expect(result.length).to eq(expected_count)
        end
      end
    end

    describe 'when the user is not authorized to create role documents' do
      before { Application.stub('clients.authorization', FalseAuthorizer.new) }

      it 'raises a ::Platform::GraphQL::Errors::ExecutionError with a not_authorized error' do
        expect { subject }.to raise_error(::Platform::GraphQL::Errors::ExecutionError, "Not authorized")
      end
    end
  end
end
