# frozen_string_literal: true
require 'cht-lib-fabric/contexts/request'
require 'cht-lib-fabric/crns'
require 'cht-lib-fabric/crns/tenant'

require 'role_management/models'
require 'role_management/graphql/schema'

require 'role_management/helpers/flow_down_helper'

RSpec.describe ::RoleManagement::GraphQL::Schema do

  let(:customer_id) { 1 }
  let(:user_id) { 17 }
  let(:organization_id) { 17 }
  let(:customer_crn) do
    ::Platform::CRNs::Tenant.new(customer: customer_id)
  end
  let(:user_crn) do
    ::Platform::CRNs::User.new(customer: customer_id, user: 17)
  end

  # Create a user defined role document to delete
  let(:role_document) {
    FactoryBot.create(:role_document,
                      name: "RoleDocument 1",
                      customer_crn: customer_crn.to_s,
                      created_by_crn: user_crn.to_s,
                      updated_by_crn: user_crn.to_s,
                      system_defined: false,
                      is_deleted: false,
                      organization_id: organization_id)
  }

  let(:role_document_crn) { "crn:#{customer_id}:role-document/#{role_document.id}" }

  let(:request) do
    req = ::Platform::Contexts::Request.new
    req.client_ip = IPAddr.new('1.1.1.1')
    req.user_agent = 'Mozilla/4.0 WebTV/2.6 (compatible; MSIE 4.0)'
    req.current_customer = customer_id
    req.current_user = user_id
    req.current_organization = organization_id
    req
  end

  let!(:variables) { nil }

  before { Application.stub('clients.authorization', TrueAuthorizer.new) }
  after { Application.unstub('clients.authorization') }

  let(:jwt_subject) { "crn:1:user/1234" }
  let(:jwt_claims) { { sub: jwt_subject } }

  before do
    allow(request).to receive(:jwt_claims).and_return(jwt_claims)
  end

  before(:each) do
    allow_any_instance_of(::RoleManagement::Helpers::FlowDownHelper).to\
      receive(:enable_flow_down).and_return([])
  end


  subject do
    RoleManagement::GraphQL::Schema.execute(
        graphql_query,
        context: { request: request },
        variables: variables
    )
  end

  describe "node query" do
    let(:graphql_query) do
      <<~GRAPHQL
        query($id: ID!) {
          node(id: $id) {
            #{node_fragment}
          }
        }
      GRAPHQL
    end

    before(:each) do
      allow_any_instance_of(::RoleManagement::Helpers::FlowDownHelper).to\
      receive(:is_flow_down_item?).and_return(false)
    end

    describe "node query for role-document" do
      let(:variables) { { id: role_document_crn } }
      let(:node_fragment) do
        <<~GRAPHQL
          id
          ... on RoleDocument {
            name
          }
        GRAPHQL
      end

      context "when the caller is not authorized" do
        before { Application.stub('clients.authorization', FalseAuthorizer.new) }
        it 'raises returns a not authorized error to the user' do
          expect(subject["errors"]).to be_a(Array)
          expect(subject["errors"].length).to eq(1)
          expect(subject["errors"][0]["message"]).to match(/Not authorized/i)
        end
      end

      context "when invalid crn" do
        let(:variables) { { id: "crn:foo:role-document/1" } }
        it 'raises returns a invalid crn error to the user' do
          puts(subject.to_h.with_indifferent_access)
          expect(subject["errors"]).to be_a(Array)
          expect(subject["errors"].length).to eq(1)
          expect(subject["errors"][0]["message"]).to eq("Invalid crn")
        end
      end

      context "when invalid crn" do
        let(:variables) { { id: "crn:#{customer_id}:role/#{role_document.id}" } }
        it 'raises returns a Invalid service error to the user' do
          puts(subject.to_h.with_indifferent_access)
          expect(subject["errors"]).to be_a(Array)
          expect(subject["errors"].length).to eq(1)
          expect(subject["errors"][0]["message"]).to eq("Invalid service")
        end
      end

      it "returns the role_document" do
        expect(subject["errors"]).to be_nil
        expect(subject["data"]["node"]["id"]).to eq(role_document_crn)
        expect(subject["data"]["node"]["name"]).to eq("RoleDocument 1")
      end
    end
  end

  describe "role-document query" do
    let(:graphql_query) do
      <<~GRAPHQL
        query roleDocuments {
          roleDocuments(last : 1 ) {
            edges {
              node {
                id 
                name
                description
                organizationId
                restricted
                createdByCRN
                updatedByCRN
              }
            } 
          }
        }
      GRAPHQL
    end

    before(:each) do
      allow_any_instance_of(::RoleManagement::Helpers::FlowDownHelper).to\
      receive(:get_flow_down_items).and_return([])
    end

    describe "role-document query" do
      let(:variables) { nil }

      before(:each) do
        FactoryBot.create(:role_document,
                          name: "RoleDocument 1",
                          customer_crn: customer_crn.to_s,
                          created_by_crn: user_crn.to_s,
                          updated_by_crn: user_crn.to_s,
                          system_defined: false,
                          is_deleted: false,
                          organization_id: organization_id)
      end

      it "returns the last role_document" do
        expect(subject["errors"]).to be_nil
        puts("subject : #{subject.to_h.with_indifferent_access}")
        expect(subject["data"]["roleDocuments"]['edges'][0]["node"]["name"]).to eq("RoleDocument 1")
        expect(subject["data"]["roleDocuments"]['edges'][0]["node"]["organizationId"]).to eq("crn:#{customer_id}:organization/#{organization_id}")
      end
    end
  end

  describe "user group rolebinding nodes" do
    let(:graphql_query) do
      <<~GRAPHQL
      query UserGroupRoleBindings {
        userGroupRoleBindings {
          nodes {
            id
            userGroupId
            organizationId
            organizationIds
            userGroupId
            createdById
            roleDocument {
              id
              name
              permissions {
                edges {
                  node {
                    id
                  }
                }
              }
            }
            updatedAt
            updatedById
          }
        }
      }
      GRAPHQL
    end

    before(:each) do
      allow_any_instance_of(::RoleManagement::Helpers::FlowDownHelper).to\
      receive(:get_flow_down_items).and_return([])
    end

    describe "nodes role-document query" do
      let(:variables) { nil }

      let(:organization_crn) { ::Platform::CRNs::Organization.new(customer: customer_id,
        details: organization_id.to_s)
      }
 
      let!(:role_binding) do
        FactoryBot.create(:user_group_role_binding,
                        customer_crn: customer_crn.to_s,
                        organization_crn: organization_crn.to_s,
                        created_by_crn: user_crn.to_s,
                        updated_by_crn: user_crn.to_s,
                        role_document: role_document)
      end

      it "returns the role document node" do
        expect(subject["errors"]).to be_nil
        puts("subject : #{subject.to_h.with_indifferent_access}")
        expect(subject["data"]["userGroupRoleBindings"]['nodes'][0]["id"]).to eq("crn:#{customer_id}:role-mgmt/ugrb:#{role_binding.id}")
        expect(subject["data"]["userGroupRoleBindings"]['nodes'][0]["roleDocument"]).to_not be_nil
        expect(subject["data"]["userGroupRoleBindings"]['nodes'][0]["roleDocument"]["id"]).to eq("crn:#{customer_id}:role-document/#{role_document.id}")
      end
    end
  end
end
