# frozen_string_literal: true

require 'role_management/models/privileges'

RSpec.describe ::RoleManagement::Models::Privileges do
  let!(:priv_container) { Class.new { include RoleManagement::Models::Privileges }.new }
  let!(:all_privileges) { priv_container.all_privileges }

  describe "all_privileges" do
    it "should be an array of two element tuples" do
      expect(all_privileges.all? do |tuple|
        ret = tuple.is_a?(Array) &&
          tuple.count == 2 &&
          tuple[0].is_a?(Symbol) &&
          tuple[1].is_a?(Symbol)
        logger.error("Bad privilege tuple: #{tuple}") unless ret
        ret
      end).to be true
    end

    it "should have the correct size" do
      expect(all_privileges.size).to be 3225 # Add permissions for Forecast Snapshot
    end
  end

  describe "builtin_admin_user_privileges" do
    let(:privileges) { priv_container.builtin_admin_privileges }
    it "should contain all privileges" do
      expect(privileges).to match(all_privileges)
    end
  end

  describe "builtin_standard_user_privileges" do
    let!(:privileges) { priv_container.builtin_standard_user_privileges }

    it "should contain whitelisted crud privileges" do
      RoleManagement::Models::Privileges::STANDARD_USER_ALLOWLIST_PRIVILEGES.each do |priv|
        expect(privileges).to include(priv), "standard user should have #{priv.inspect} privilege"
      end
    end

    it "should not contain blacklisted crud privileges" do
      RoleManagement::Models::Privileges::STANDARD_USER_DENYLIST_PRIVILEGES.each do |priv|
        expect(privileges).not_to include(priv), "standard user should not have #{priv.inspect} privilege"
      end
    end

    it "other than whitelist it should only contain whitelisted action privileges" do
      expect((privileges - RoleManagement::Models::Privileges::STANDARD_USER_ALLOWLIST_PRIVILEGES)
               .map(&:first)
               .uniq).to match_array(RoleManagement::Models::Privileges::STANDARD_USER_ALLOWLIST_ACTIONS)
    end
  end

  describe "oracle_cost_report_privileges" do
    let(:privileges) { priv_container.oracle_cost_report_privileges }
    it "should contain oracle cost privileges" do
      expect(privileges.count).to eq(1)
    end
  end

  describe "container_report_privileges" do
    let(:privileges) { priv_container.container_report_privileges }
    it "should contain container report privileges" do
      expect(privileges.count).to eq(10)
    end
  end

  describe "cost_anomaly_detection_privileges" do
    let(:privileges) { priv_container.cost_anomaly_detection_privileges }
    it "should contain cost anomaly detection privileges" do
      expect(privileges.count).to eq(3)
    end
  end

  describe "azure_account_assignment_privileges" do
    let(:standard_user_privileges) { priv_container.builtin_standard_user_privileges }
    let(:admin_privileges) { priv_container.builtin_admin_privileges }
    let(:power_user_privileges) { priv_container.builtin_power_user_privileges }
    let(:customer_admin_privileges) { priv_container.builtin_customer_admin_privileges }
    let(:azure_account_assignment_create_privilege) { %i[create azure_account_assignment] }
    let(:azure_account_assignment_read_privilege) { %i[read azure_account_assignment] }
    let(:azure_account_assignment_update_privilege) { %i[update azure_account_assignment] }
    let(:azure_account_assignment_delete_privilege) { %i[delete azure_account_assignment] }

    it "should be available for power users, admins, and customer admins" do
      [power_user_privileges, customer_admin_privileges, admin_privileges].map do |privileges|
        expect(privileges).to include(azure_account_assignment_create_privilege)
        expect(privileges).to include(azure_account_assignment_read_privilege)
        expect(privileges).to include(azure_account_assignment_update_privilege)
        expect(privileges).to include(azure_account_assignment_delete_privilege)
      end
    end

    it "should not be available for standard users" do
      expect(standard_user_privileges).to_not include(azure_account_assignment_create_privilege)
      expect(standard_user_privileges).to include(azure_account_assignment_read_privilege)
      expect(standard_user_privileges).to_not include(azure_account_assignment_update_privilege)
      expect(standard_user_privileges).to_not include(azure_account_assignment_delete_privilege)
    end
  end

  describe "flex_dataset_privileges" do
    let(:standard_user_privileges) { priv_container.builtin_standard_user_privileges }
    let(:admin_privileges) { priv_container.builtin_admin_privileges }
    let(:power_user_privileges) { priv_container.builtin_power_user_privileges }
    let(:customer_admin_privileges) { priv_container.builtin_customer_admin_privileges }

    let(:containers_read_privilege) { %i[read kubernetes_containers_usage_flexreport_dataset] }
    let(:nodes_read_privilege) { %i[read kubernetes_nodes_usage_flexreport_dataset] }
    let(:aws_k8s_cost_read_privilege) { %i[read aws_kubernetes_cost_dataset] }
    let(:azure_k8s_cost_read_privilege) { %i[read azure_kubernetes_cost_dataset] }

    let(:containers_create_privilege) { %i[create kubernetes_containers_usage_flexreport_dataset] }
    let(:nodes_create_privilege) { %i[create kubernetes_nodes_usage_flexreport_dataset] }
    let(:aws_k8s_cost_create_privilege) { %i[create aws_kubernetes_cost_dataset] }
    let(:azure_k8s_cost_create_privilege) { %i[create azure_kubernetes_cost_dataset] }

    it "should be only read for all kubernetes datasets for all users" do
      [standard_user_privileges, power_user_privileges, customer_admin_privileges, admin_privileges].map do |privileges|
        expect(privileges).to include(containers_read_privilege)
        expect(privileges).to include(nodes_read_privilege)
        expect(privileges).to include(aws_k8s_cost_read_privilege)
        expect(privileges).to include(azure_k8s_cost_read_privilege)

        expect(privileges).to_not include(containers_create_privilege)
        expect(privileges).to_not include(nodes_create_privilege)
        expect(privileges).to_not include(aws_k8s_cost_create_privilege)
        expect(privileges).to_not include(azure_k8s_cost_create_privilege)
      end
    end
  end

  describe "flex_view_privileges" do
    let(:standard_user_privileges) { priv_container.builtin_standard_user_privileges }
    let(:admin_privileges) { priv_container.builtin_admin_privileges }
    let(:power_user_privileges) { priv_container.builtin_power_user_privileges }
    let(:customer_admin_privileges) { priv_container.builtin_customer_admin_privileges }

    let(:containers_read_privilege) { %i[read kubernetes_containers_usage_view] }
    let(:containers_export_privilege) { %i[export kubernetes_containers_usage_view] }
    let(:nodes_read_privilege) { %i[read kubernetes_nodes_usage_view] }
    let(:nodes_export_privilege) { %i[export kubernetes_nodes_usage_view] }
    let(:aws_k8s_cost_read_privilege) { %i[read azure_kubernetes_cost_view] }
    let(:aws_k8s_cost_export_privilege) { %i[export azure_kubernetes_cost_view] }
    let(:azure_k8s_cost_read_privilege) { %i[read aws_kubernetes_cost_view] }
    let(:azure_k8s_cost_export_privilege) { %i[export aws_kubernetes_cost_view] }

    let(:containers_create_privilege) { %i[create kubernetes_containers_usage_view] }
    let(:nodes_create_privilege) { %i[create kubernetes_nodes_usage_view] }
    let(:aws_k8s_cost_create_privilege) { %i[create azure_kubernetes_cost_view] }
    let(:azure_k8s_cost_create_privilege) { %i[create aws_kubernetes_cost_view] }

    it "should include read for all kubernetes views for all users" do
      [standard_user_privileges, power_user_privileges, customer_admin_privileges, admin_privileges].map do |privileges|
        expect(privileges).to include(containers_read_privilege)
        expect(privileges).to include(nodes_read_privilege)
        expect(privileges).to include(aws_k8s_cost_read_privilege)
        expect(privileges).to include(azure_k8s_cost_read_privilege)
        expect(privileges).to include(containers_export_privilege)
        expect(privileges).to include(nodes_export_privilege)
        expect(privileges).to include(aws_k8s_cost_export_privilege)
        expect(privileges).to include(azure_k8s_cost_export_privilege)

        expect(privileges).to_not include(containers_create_privilege)
        expect(privileges).to_not include(nodes_create_privilege)
        expect(privileges).to_not include(aws_k8s_cost_create_privilege)
        expect(privileges).to_not include(azure_k8s_cost_create_privilege)
      end
    end
  end
end
