# frozen_string_literal: true

require 'role_management/stores/role_documents_directory'
require 'cht-lib-fabric/crn'
require 'cht-lib-fabric/crns'
require 'cht-lib-fabric/crns/tenant'
require 'role_management/models'
require 'role_management/errors'

RSpec.describe ::RoleManagement::Stores::RoleDocumentsDirectory do
  let(:directory) {
    Application['role_management.stores.role_documents_directory']
  }
  let(:tenant_crn) { "crn:1:tenant/1" }
  let(:user_crn) { "crn:1:user/3871" }
  let(:organization_id) { "crn:1:organization/24" }

  describe '#create_custom_role_document' do
    let(:fake_name) { "Fake role doc" }
    let(:fake_description) { "Fake description for role doc" }
    let(:valid_permissions) { [ {"action" => "read", "subject" => "custom_report"},
                                {"action" => "update", "subject" => "custom_report"} ] }

    describe 'creating a full role document' do
      it 'returns a role document model with all attributes' do
        result = directory.create_custom_role_document(name: fake_name,
                                                       description: fake_description,
                                                       restricted: false,
                                                       customer_crn: tenant_crn,
                                                       user_crn: user_crn,
                                                       permissions: valid_permissions,
                                                       organization: organization_id)
        expect(result.id).not_to be_nil
        expect(result.name).to eq(fake_name)
        expect(result.description).to eq(fake_description)
        expect(result.restricted).to be false
        expect(result.system_defined).to be false
        expect(result.created_by).to eq(user_crn)
        expect(result.updated_by).to eq(user_crn)
        expect(result.permissions).not_to be_nil
        expect(result.permissions.count).to eq(valid_permissions.count)
      end
    end

    context 'partial creation' do
      describe 'creating a role document without permissions' do
        it 'returns a role document model with all attributes and nil permissions' do
          result = directory.create_custom_role_document(name: fake_name,
                                                         description: fake_description,
                                                         restricted: false,
                                                         customer_crn: tenant_crn,
                                                         user_crn: user_crn,
                                                         organization: organization_id)
          expect(result.id).not_to be_nil
          expect(result.name).to eq(fake_name)
          expect(result.description).to eq(fake_description)
          expect(result.restricted).to be false
          expect(result.system_defined).to be false
          expect(result.created_by).to eq(user_crn)
          expect(result.updated_by).to eq(user_crn)
          expect(result.permissions).not_to be_nil
          expect(result.permissions).to be_empty
        end
      end

      describe 'create a role document without a description' do
        it 'returns a role document model with all attributes and nil description' do
          result = directory.create_custom_role_document(name: fake_name,
                                                         customer_crn: tenant_crn,
                                                         restricted: false,
                                                         user_crn: user_crn,
                                                         organization: organization_id)
          expect(result.id).not_to be_nil
          expect(result.name).to eq(fake_name)
          expect(result.description).to be_nil
          expect(result.restricted).to be false
          expect(result.system_defined).to be false
          expect(result.created_by).to eq(user_crn)
          expect(result.updated_by).to eq(user_crn)
          expect(result.permissions).not_to be_nil
          expect(result.permissions).to be_empty
        end
      end

      describe 'create a restricted role document' do
        it 'returns a role document model with all attributes and restricted flag true' do
          result = directory.create_custom_role_document(name: fake_name,
                                                         customer_crn: tenant_crn,
                                                         restricted: true,
                                                         user_crn: user_crn,
                                                         organization: organization_id)
          expect(result.id).not_to be_nil
          expect(result.name).to eq(fake_name)
          expect(result.description).to be_nil
          expect(result.restricted).to be true
          expect(result.system_defined).to be false
          expect(result.created_by).to eq(user_crn)
          expect(result.updated_by).to eq(user_crn)
          expect(result.permissions).not_to be_nil
          expect(result.permissions).to be_empty
        end
      end
    end

    context 'exceptions' do
      describe 'creating a role document without a name' do
        it 'raises an exception' do

        end
      end

      describe 'creating a role document with invalid permissions' do
        let(:invalid_permissions) { [{"action" => "read", "subject" => "foo"}] }

        it 'raises an exception and does not persist the role document' do
          expect {
            directory.create_custom_role_document(name: fake_name,
                                                  description: fake_description,
                                                  restricted: false,
                                                  customer_crn: tenant_crn,
                                                  user_crn: user_crn,
                                                  permissions: invalid_permissions,
                                                  organization: organization_id)
          }.to raise_error(::RoleManagement::Exceptions::InvalidPermission)
        end
      end
    end
  end

  describe '#update_custom_role_document' do
    subject do
      directory.update_custom_role_document(id,
                                            name: updated_name,
                                            description: updated_description,
                                            customer_crn: tenant_crn,
                                            updated_by_crn: user_crn,
                                            permissions: updated_permissions,
                                            organization: organization_id)
    end

    let!(:existing_role_doc) { FactoryBot.create(:role_document,
                                                 name: "Existing role doc",
                                                 customer_crn: tenant_crn,
                                                 updated_by_crn: user_crn,
                                                 system_defined: false,
                                                 is_deleted: false,
                                                 organization_id: organization_id) }

    let(:updated_name) { "Fake role doc" }
    let(:updated_description) { "Fake description for role doc" }
    let(:updated_permissions) { [ {"action" => "read", "subject" => "custom_report"},
                                  {"action" => "update", "subject" => "custom_report"},
                                  {"action" => "delete", "subject" => "custom_report"} ] }

    context 'exceptions' do
      describe 'when does not exist' do
        let(:id) { 999999999 } # fake id

        it 'raises an ::ActiveRecord::RecordNotFound exception' do
          expect {
            subject
          }.to raise_error(::ActiveRecord::RecordNotFound)
        end
      end

      describe 'when a system role document' do
        let(:id) { ::RoleManagement::Stores::RoleDocument.where(system_defined: true).first.id }

        it 'raises an ::ActiveRecord::RecordNotFound exception' do
          expect {
            subject
          }.to raise_error(::ActiveRecord::RecordNotFound)
        end
      end

      describe 'when given an invalid permission' do
        let(:id) { existing_role_doc.id }
        let(:updated_permissions) { [{"action" => "read", "subject" => "foo"}] }

        it 'raises an ::RoleManagement::Exceptions::InvalidPermission exception' do
          expect {
            subject
          }.to raise_error(::RoleManagement::Exceptions::InvalidPermission)
        end
      end
    end

    context 'full update' do
      let(:id) { existing_role_doc.id }

      describe 'when given values for all update fields' do
        it 'returns a role document model with all updated attributes' do
          result = subject

          expect(result.name).to eq(updated_name)
          expect(result.description).to eq(updated_description)
          expect(result.system_defined).to be false
          expect(result.created_by).to eq(user_crn)
          expect(result.updated_by).to eq(user_crn)
          expect(result.permissions.count).to eq(updated_permissions.count)
        end
      end
    end

    context 'partial update' do
      let(:id) { existing_role_doc.id }

      describe 'only update the name' do
        let(:updated_name) { "Update the name" }
        let(:updated_permissions) { nil }
        let(:updated_description) { nil }

        it 'returns a role document model with everything the same except the name' do
          result = subject

          expect(result.name).to eq(updated_name)
          expect(result.description).to eq(existing_role_doc.description)
          expect(result.system_defined).to be false
          expect(result.created_by).to eq(user_crn)
          expect(result.updated_by).to eq(user_crn)
          expect(result.permissions.count).to eq(existing_role_doc.permissions.count)
        end
      end

      describe 'only update the description' do
        let(:updated_name) { nil }
        let(:updated_permissions) { nil }
        let(:updated_description) { "Update the description" }

        it 'returns a role document model with everything the same except the description' do
          result = subject

          expect(result.name).to eq(existing_role_doc.name)
          expect(result.description).to eq(updated_description)
          expect(result.system_defined).to be false
          expect(result.created_by).to eq(user_crn)
          expect(result.updated_by).to eq(user_crn)
        end
      end

      describe 'only updating permissions' do
        let(:updated_name) { nil }
        let(:updated_description) { nil }

        describe 'when passed empty permissions' do
          let(:updated_permissions) { [] }

          it 'returns a role document model with empty permissions' do
            result = subject

            expect(result.name).to eq(existing_role_doc.name)
            expect(result.description).to eq(existing_role_doc.description)
            expect(result.system_defined).to be false
            expect(result.created_by).to eq(user_crn)
            expect(result.updated_by).to eq(user_crn)
            expect(result.permissions).to be_empty
          end
        end

        describe 'when passed updated permissions' do
          let(:updated_permissions) { [{"action" => "read", "subject" => "custom_report"}] }

          it 'returns a role document model with updated permission and updated date' do
            #adding sleep in order to have difference in created and updated time
            sleep(5)
            result = subject

            expect(result.name).to eq(existing_role_doc.name)
            expect(result.description).to eq(existing_role_doc.description)
            expect(result.system_defined).to be false
            expect(result.created_by).to eq(user_crn)
            expect(result.updated_by).to eq(user_crn)
            expect(result.permissions.count).to eq(updated_permissions.count)
            expect(result.updated_at).not_to eq(result.created_at)
          end
        end
      end
    end

    context 'user context' do
      let(:id) { existing_role_doc.id }

      describe 'when updated by the same user' do
        it 'returns a role document model with the same updated_by field and created_at unchanged' do
          result = subject

          expect(result.created_by).to eq(user_crn)
          expect(result.updated_by).to eq(user_crn)
        end
      end

      describe 'when updated by a different user' do
        let(:user_crn) { "crn:1:user/2"}
        let(:creator_crn) { existing_role_doc.created_by_crn }

        it 'returns a role document model with a different updated_by field and created_at unchanged' do
          result = subject

          expect(result.created_by).to eq(creator_crn)
          expect(result.updated_by).to eq(user_crn)
        end
      end
    end
  end

  describe '#destroy_role_document' do
    let!(:role_document) { FactoryBot.create(:role_document,
                                              name: "To be deleted",
                                              customer_crn: tenant_crn,
                                              created_by_crn: user_crn,
                                              updated_by_crn: user_crn,
                                              system_defined: false,
                                              is_deleted: false,
                                              organization_id: organization_id) }
    describe 'when deleting a system defined record' do
      it 'returns ActiveRecord::RecordNotFound' do
        expect {
          directory.destroy_role_document(1, "crn::tenant/0", organization_id)
        }.to raise_error(ActiveRecord::RecordNotFound)
      end
    end

    describe 'when deleting an invalid record' do
      it 'returns ActiveRecord::RecordNotFound' do
        expect {
          directory.destroy_role_document(124124, tenant_crn, organization_id)
        }.to raise_error(ActiveRecord::RecordNotFound)
      end
    end

    describe 'when deleting a valid record' do
      it 'returns destroyed record' do
        record = directory.destroy_role_document(role_document.id, tenant_crn, organization_id)
        expect(record.id).to eq(role_document.id)
      end
    end
  end

  describe 'user_group_role_bindings_by_user_group' do

    let(:user_group_crn_1) { "crn:1:user-group/1" }
    let(:user_group_crn_2) { "crn:1:user-group/2" }

    let!(:existing_role_doc_1) { FactoryBot.create(:role_document,
                                                 name: "Existing role doc 1",
                                                 customer_crn: tenant_crn,
                                                 updated_by_crn: user_crn,
                                                 system_defined: false,
                                                 is_deleted: false,
                                                 organization_id: organization_id) }
    let!(:existing_role_doc_2) { FactoryBot.create(:role_document,
                                                 name: "Existing role doc 2",
                                                 customer_crn: tenant_crn,
                                                 updated_by_crn: user_crn,
                                                 system_defined: false,
                                                 is_deleted: false,
                                                 organization_id: organization_id) }

    let!(:ugrb_1) { FactoryBot.create(:user_group_role_binding,
                                    customer_crn: tenant_crn,
                                    organization_crn: organization_id,
                                    created_by_crn: user_crn,
                                    updated_by_crn: user_crn,
                                    role_document: existing_role_doc_1,
                                    user_group_crn: user_group_crn_1) }

    let!(:ugrb_2) { FactoryBot.create(:user_group_role_binding,
                                      customer_crn: tenant_crn,
                                      organization_crn: organization_id,
                                      created_by_crn: user_crn,
                                      updated_by_crn: user_crn,
                                      role_document: existing_role_doc_2,
                                      user_group_crn: user_group_crn_1) }

    let!(:ugrb_3) { FactoryBot.create(:user_group_role_binding,
                                      customer_crn: tenant_crn,
                                      organization_crn: organization_id,
                                      created_by_crn: user_crn,
                                      updated_by_crn: user_crn,
                                      role_document: existing_role_doc_1,
                                      user_group_crn: user_group_crn_2) }

    describe 'when user-group associated ugrb exists' do
      it 'fetches associated user-group-role-binding' do
        result = directory.user_group_role_bindings_by_user_group(user_group_crn: user_group_crn_1)
        expect(result.size).to eq(2)
      end

      it 'fetches associated user-group-role-binding' do
        result = directory.user_group_role_bindings_by_user_group(user_group_crn: user_group_crn_2)
        expect(result.size).to eq(1)
      end
    end

    describe 'when user-group associated ugrb does not exists' do
      let(:user_group_crn_3) { "crn:1:user-group/3" }
      it 'returns original list of ugrb' do
        result = directory.user_group_role_bindings_by_user_group(user_group_crn: user_group_crn_3)
        expect(result.size).to eq(0)
      end
    end
  end
end
